#include "Box.h"

Box::Box()
{
}
Box::~Box()
{
}
//returns the dist if it intersects, -1 otherwise
float Box::Intersect(Vector rayOrg, Vector rayDir, Vector &norm)
{
	//P0 = rayOrg, d0 = rayDir

	double tmin=0.0;
	double tmax=LARGE_FLOAT;
	double interval, temp;
	int tminPlane = -1;
	int tmaxPlane = -1;
	for(int i=0; i < 6 && tmin <= tmax; ++i)
	{
		temp = rayDir * planes[i].normal;
		if(temp < 0)
		{
			interval=-((rayOrg - planes[i].point) * planes[i].normal)/(rayDir * planes[i].normal);
			tmin = max(interval, tmin);
			if(tmin == interval)
				tminPlane = i;
		}
		else if(temp > 0)
		{
			interval=-((rayOrg - planes[i].point) * planes[i].normal)/(rayDir * planes[i].normal);
			tmax = min(interval, tmax);
			if(tmax == interval)
				tmaxPlane = i;
		}
		else if((rayOrg - planes[i].point) * planes[i].normal > 0)
		{
			tmin = tmax + 1.0;
			tminPlane = i;
		}
	}
	if(tmin > tmax)
		return(-1.0f);
	if(tmin > 0.0)
	{
		norm = planes[tminPlane].normal;
		return((float)tmin);
	}
	else if(tmin == 0.0)
	{
		norm = planes[tmaxPlane].normal;
		return((float)tmax);
	}
	return(-1.0f);
}
void Box::CalculatePlanes()
{
	//frontNorm, backNorm, leftNorm, rightNorm, topNorm, bottomNorm
	planes[0].normal = length ^ height;
	planes[1].normal = -planes[0].normal;
	planes[2].normal = height ^ width;
	planes[3].normal = -planes[2].normal;
	planes[4].normal = width ^ length;
	planes[5].normal = -planes[4].normal;

	planes[0].point = corner;
	planes[1].point = corner + width;
	planes[2].point = corner;
	planes[3].point = corner + length;
	planes[4].point = corner;
	planes[5].point = corner + height;
}
void Box::SetCorner(Vector c)
{
	corner = c;
}
Vector Box::GetCorner()
{
	return(corner);
}
void Box::SetLength(Vector c)
{
	length = c;
}
Vector Box::GetLength()
{
	return(length);
}
void Box::SetWidth(Vector c)
{
	width = c;
}
Vector Box::GetWidth()
{
	return(width);
}
void Box::SetHeight(Vector c)
{
	height = c;
}
Vector Box::GetHeight()
{
	return(height);
}