#include "Ellipsoid.h"

Ellipsoid::Ellipsoid()
{
}
Ellipsoid::~Ellipsoid()
{
}
float Ellipsoid::Intersect(Vector rayOrg, Vector rayDir, Vector &norm)
{
	/*
	MI = matrix inverse
	a = |MI * d0|^2
	b = 2[(MI * (P0 - C))  dot  (MI * d0)]
	c = |MI * (P0 - C)|^2 - 1
	P0 = rayOrg, d0 = rayDir
	C = center, r = radius
	discriminant = b^2 - 4ac
	*/
	Vector temp = (rayOrg - center);
	float a, b, c, det, tplus, tminus, ret;

	a = (axisInverse * rayDir).LengthSq();
	b = 2 * ((axisInverse * temp) * (axisInverse * rayDir));
	c = (axisInverse * temp).LengthSq() - 1.0f;
	det = pow(b, 2) - (4*a*c);
	if(det < 0)
		return(-1.0f);
	else
	{
		tplus = (-b + sqrt(det))/(2*a);
		if(tplus < 0)
			return(-1.0f);
		else
		{
			tminus = (-b - sqrt(det))/(2*a);
			if(tminus < 0)
			{
				Matrix3x3 aiTranspose = axisInverse.Transpose();

				ret = (rayDir * tplus).Length();
				Vector interPoint = rayOrg + (rayDir * tplus);

				norm = (aiTranspose * axisInverse * (interPoint - center));
				return(tplus);
			}
			else
			{
				Matrix3x3 aiTranspose = axisInverse.Transpose();

				ret = (rayDir * tminus).Length();
				Vector interPoint = rayOrg + (rayDir * tminus);

				norm = (aiTranspose * axisInverse * (interPoint - center));
				return(tminus);
			}
		}
	}
	return(-1.0f);
}
void Ellipsoid::SetCenter(Vector c)
{
	center = c;
}
Vector Ellipsoid::GetCenter()
{
	return(center);
}
void Ellipsoid::SetAxisX(Vector c)
{
	axisx = c;
}
Vector Ellipsoid::GetAxisX()
{
	return(axisx);
}
void Ellipsoid::SetAxisY(Vector c)
{
	axisy = c;
}
Vector Ellipsoid::GetAxisY()
{
	return(axisy);
}
void Ellipsoid::SetAxisZ(Vector c)
{
	axisz = c;
}
Vector Ellipsoid::GetAxisZ()
{
	return(axisz);
}
void Ellipsoid::CalculateAxisInverse()
{
	axisInverse._matrix[0][0] = axisx.x; axisInverse._matrix[1][0] = axisx.y; axisInverse._matrix[2][0] = axisx.z;
	axisInverse._matrix[0][1] = axisy.x; axisInverse._matrix[1][1] = axisy.y; axisInverse._matrix[2][1] = axisy.z;
	axisInverse._matrix[0][2] = axisz.x; axisInverse._matrix[1][2] = axisz.y; axisInverse._matrix[2][2] = axisz.z;
	axisInverse = axisInverse.Invert();
}