#include "Polygon.h"

Poly::Poly()
{
}
Poly::~Poly()
{
}
float Poly::Intersect(Vector rayOrg, Vector rayDir, Vector &norm)
{
	//P0 = rayOrg, d0 = rayDir

	int length = GetNumVerts() - 2;
	float time;

	if(length < 1)
		return(-1.0f);
	Vector side1, side2, tnorm, temp;
	for(int i=0; i < length; ++i)
	{
		side1 = vertices[i + 1] - vertices[0];
		side2 = vertices[i + 2] - vertices[0];
		tnorm = side1 ^ side2;
		time = -((rayOrg - vertices[i]) * tnorm)/(rayDir * tnorm);
		if(time < 0)
			continue;
		Vector P = rayOrg + (rayDir * time);
		//now compute alpha and beta
		float alpha, beta, det;
		float a11, a12, a22, b1, b2;
		a11 = side2.LengthSq();
		a12 = -(side1 * side2);
		a22 = side1.LengthSq();
		temp = (P - vertices[0]);
		b1 = (temp * side1);
		b2 = (temp * side2);
		det = 1.0f/((a22 * a11) - pow(side1 * side2, 2));
		alpha = ((a11 * b1) + (a12 * b2)) * det;
		beta = ((a12 * b1) + (a22 * b2)) * det;
		
		if(alpha >= -EPSILON && beta >= -EPSILON)
		{
			if(alpha + beta <= (1.0f + EPSILON))
			{
				norm = tnorm;
				return(time);
			}
		}
	}
	return(-1.0f);
}
void Poly::SetNumVerts(unsigned int c)
{
	numVerts = c;
}
unsigned int Poly::GetNumVerts()
{
	return(numVerts);
}
void Poly::AddVert(Vector v)
{
	vertices.push_back(v);
}
Vector Poly::GetVert(unsigned int num)
{
	return(vertices[num]);
}