#include "matrix.h"
#include <stdlib.h>
#include <assert.h>


Matrix3x3 operator *( const Matrix3x3 & lhs, const Matrix3x3 rhs )
{
	Matrix3x3 value;
	for( int row = 0; row < 3; ++row )
		for( int col = 0; col < 3; ++col )
			for( int i = 0; i < 3; ++i )
			{
				value._matrix[row][col] += lhs._matrix[row][i] * rhs._matrix[i][col];
			}
	return (value);
}	

Matrix3x3 operator *( const Matrix3x3 & lhs, const float scaler )
{
	Matrix3x3 value;

	for( int row = 0; row < 3; ++row )
		for( int col = 0; col < 3; ++col )
		{
			value._matrix[col][row] = lhs._matrix[col][row] * scaler;
		}
		return (value);
}


Matrix3x3::Matrix3x3( void )
{
	for( int i = 0; i < 3; ++i )
		for( int j = 0; j < 3; ++j )
		{
			if( i != j )
				_matrix[i][j] = 0.0f;
			else
				_matrix[i][j] = 1.0f;
		}
}

Matrix3x3::Matrix3x3( const Matrix3x3 & rhs )
{
	*this = rhs;
}

Matrix3x3::Matrix3x3( float v00, float v01, float v02,
											float v10, float v11, float v12,
											float v20, float v21, float v22 )
{
	/*Untested, swapped indices to normal 2/11/07*/
	_matrix[0][0] = v00;	_matrix[0][1] = v01;	_matrix[0][2] = v02;
	_matrix[1][0] = v10;	_matrix[1][1] = v11;	_matrix[1][2] = v12;
	_matrix[2][1] = v20;	_matrix[2][1] = v21;	_matrix[2][2] = v22;
}

Matrix3x3::~Matrix3x3( void )
{
}

Matrix3x3 & Matrix3x3::operator =( const Matrix3x3 & rhs )
{
	for( int i = 0; i < 3; ++i )
		for( int j = 0; j < 3; ++j )
		{
			_matrix[i][j] = rhs._matrix[i][j];
		}
		return *this;
}

bool Matrix3x3::operator ==( const Matrix3x3 & rhs )
{
	for( int row = 0; row < 3; ++row )
		for( int col = 0; col < 3; ++col )
		{
			if( this->_matrix[col][row] != rhs._matrix[col][row] )
				return false;
		}
		return true;
}


void Matrix3x3::operator *=( const Matrix3x3 & rhs )
{
	*this = (*this) * rhs;
}

void Matrix3x3::operator *=( const float scaler )
{
	for( int i = 0; i < 3; ++i )
		for( int j = 0; j < 3; ++j )
		{
			_matrix[i][j] *= scaler;
		}
}

Matrix3x3 Matrix3x3::operator *( const float scaler )
{
	Matrix3x3 value = *this;
	value *= scaler;
	return value;
}

Vector Matrix3x3::operator *( const Vector & v )
{
	Vector value;
	/*Untested, swapped indices to normal 2/11/07
	value.x = ( _matrix[0][0] * v.x ) + ( _matrix[1][0] * v.y ) + ( _matrix[2][0] * v.z );
	value.y = ( _matrix[0][1] * v.x ) + ( _matrix[1][1] * v.y ) + ( _matrix[2][1] * v.z );
	value.z = ( _matrix[0][2] * v.x ) + ( _matrix[1][2] * v.y ) + ( _matrix[2][2] * v.z );
	*/
	value.x = ( _matrix[0][0] * v.x ) + ( _matrix[0][1] * v.y ) + ( _matrix[0][2] * v.z );
	value.y = ( _matrix[1][0] * v.x ) + ( _matrix[1][1] * v.y ) + ( _matrix[1][2] * v.z );
	value.z = ( _matrix[2][0] * v.x ) + ( _matrix[2][1] * v.y ) + ( _matrix[2][2] * v.z );
	return value;
}

void Matrix3x3::BuildRotation(Vector axis, float angle )
{	
	axis.Normalize();
	float x = axis.x;
	float y = axis.y;
	float z = axis.z;
	float c = cos(angle);
	float s = sin(angle);
	float t = 1.0f - c;
	
	/*Untested, swapped indices to normal 2/11/07
	_matrix[0][0] = (t*x*x + c);   _matrix[0][1] = (t*x*y + s*z); _matrix[0][2] = (t*x*z - s*y);
	_matrix[1][0] = (t*x*y - s*z); _matrix[1][1] = (t*y*y + c);   _matrix[1][2] = (t*y*z + s*x);
	_matrix[2][0] = (t*x*z + s*y), _matrix[2][1] = (t*y*z - s*x); _matrix[2][2] = (t*z*z + c);
	*/
	_matrix[0][0] = (t*x*x + c);   _matrix[1][0] = (t*x*y + s*z); _matrix[2][0] = (t*x*z - s*y);
	_matrix[0][1] = (t*x*y - s*z); _matrix[1][1] = (t*y*y + c);   _matrix[2][1] = (t*y*z + s*x);
	_matrix[0][2] = (t*x*z + s*y), _matrix[1][2] = (t*y*z - s*x); _matrix[2][2] = (t*z*z + c);
}

float Matrix3x3::Determinant()
{
	return (_matrix[0][0]*(_matrix[1][1]*_matrix[2][2]-_matrix[1][2]*_matrix[2][1])
		- _matrix[0][1]*(_matrix[1][0]*_matrix[2][2]-_matrix[1][2]*_matrix[2][0])
		+ _matrix[0][2]*(_matrix[1][0]*_matrix[2][1]-_matrix[1][1]*_matrix[2][0]));

}
Matrix3x3 Matrix3x3::Invert()
{
	float det = Determinant();
	Matrix3x3 result;

	if (fabs(det) < EPSILON)
	{
		result = Matrix3x3();
		return result;
	}
	else
	{
		result._matrix[0][0] = _matrix[1][1]*_matrix[2][2] - _matrix[1][2]*_matrix[2][1];
		result._matrix[0][1] = _matrix[2][1]*_matrix[0][2] - _matrix[2][2]*_matrix[0][1];
		result._matrix[0][2] = _matrix[0][1]*_matrix[1][2] - _matrix[0][2]*_matrix[1][1];
		result._matrix[1][0] = _matrix[1][2]*_matrix[2][0] - _matrix[1][0]*_matrix[2][2];
		result._matrix[1][1] = _matrix[2][2]*_matrix[0][0] - _matrix[2][0]*_matrix[0][2];
		result._matrix[1][2] = _matrix[0][2]*_matrix[1][0] - _matrix[0][0]*_matrix[1][2];
		result._matrix[2][0] = _matrix[1][0]*_matrix[2][1] - _matrix[1][1]*_matrix[2][0];
		result._matrix[2][1] = _matrix[2][0]*_matrix[0][1] - _matrix[2][1]*_matrix[0][0];
		result._matrix[2][2] = _matrix[0][0]*_matrix[1][1] - _matrix[0][1]*_matrix[1][0];

		return (result * (1.0f/det));
	}
}

Matrix3x3 Matrix3x3::Transpose()
{
	Matrix3x3 transpose;
	for(int i=0; i < 3; ++i)
	{
		for(int j=0; j < 3; ++j)
		{
			transpose._matrix[i][j] = _matrix[j][i];
		}
	}
	return(transpose);
}


Matrix4x4 operator *( const Matrix4x4 & lhs, const Matrix4x4 rhs )
{
	Matrix4x4 value;
	for( int row = 0; row < 4; ++row )
		for( int col = 0; col < 4; ++col )
			for( int i = 0; i < 4; ++i )
			{
				value._matrix[col][row] += lhs._matrix[i][row] * rhs._matrix[col][i];
			}
			return value;
}	

Matrix4x4 operator *( const Matrix4x4 & lhs, const int scaler )
{
	Matrix4x4 value;

	for( int row = 0; row < 4; ++row )
		for( int col = 0; col < 4; ++col )
		{
			value._matrix[col][row] = lhs._matrix[col][row] * scaler;
		}
		return value;
}


Matrix4x4::Matrix4x4( void )
{
	for( int i = 0; i < 4; ++i )
		for( int j = 0; j < 4; ++j )
		{
			if( i != j )
				_matrix[i][j] = 0.0f;
			else
				_matrix[i][j] = 1.0f;
		}
}

Matrix4x4::Matrix4x4( const Matrix4x4 & rhs )
{
	*this = rhs;
}

Matrix4x4::Matrix4x4( float v00, float v01, float v02, float v03,
				float v10, float v11, float v12, float v13,
				float v20, float v21, float v22, float v23,
				float v30, float v31, float v32, float v33)
{
	_matrix[0][0] = v00;	_matrix[1][0] = v01;	_matrix[2][0] = v02;	_matrix[3][0] = v03;
	_matrix[0][1] = v10;	_matrix[1][1] = v11;	_matrix[2][1] = v12;	_matrix[3][1] = v13;
	_matrix[0][2] = v20;	_matrix[1][2] = v21;	_matrix[2][2] = v22;	_matrix[3][2] = v23;
	_matrix[0][3] = v30;	_matrix[1][3] = v31;	_matrix[2][3] = v32;	_matrix[3][3] = v33;
}

Matrix4x4::~Matrix4x4( void )
{
}

Matrix4x4 & Matrix4x4::operator =( const Matrix4x4 & rhs )
{
	for( int i = 0; i < 4; ++i )
		for( int j = 0; j < 4; ++j )
		{
			_matrix[i][j] = rhs._matrix[i][j];
		}
		return *this;
}

bool Matrix4x4::operator ==( const Matrix4x4 & rhs )
{
	for( int row = 0; row < 4; ++row )
		for( int col = 0; col < 4; ++col )
		{
			if( this->_matrix[col][row] != rhs._matrix[col][row] )
				return false;
		}
		return true;
}

Matrix4x4 Matrix4x4::Inverse()
{
	Matrix4x4 out;
	float temp, det=0;
    float pos, neg;
	pos = neg = 0.0;

    temp =  _matrix[0][0] * _matrix[1][1] * _matrix[2][2];
    det += temp;
    temp =  _matrix[0][1] * _matrix[1][2] * _matrix[2][0];
    det += temp;
    temp =  _matrix[0][2] * _matrix[1][0] * _matrix[2][1];
    det += temp;
    temp = -_matrix[0][2] * _matrix[1][1] * _matrix[2][0];
    det += temp;
    temp = -_matrix[0][1] * _matrix[1][0] * _matrix[2][2];
    det += temp;
    temp = -_matrix[0][0] * _matrix[1][2] * _matrix[2][1];
    det += temp;

    /* Is the submatrix A singular? */
    if ((det == 0.0) || (abs(det / (pos - neg)) < EPSILON)) {
		assert(0);
		exit(0);
    }

    else {

        /* Calculate inverse(A) = adj(A) / det(A) */
        det = 1.0f / det;
		out._matrix[0][0] =   ( _matrix[1][1] * _matrix[2][2] -
                                 _matrix[1][2] * _matrix[2][1] )
                               * det;
        out._matrix[1][0] = - ( _matrix[1][0] * _matrix[2][2] -
                                 _matrix[1][2] * _matrix[2][0] )
                               * det;
        out._matrix[2][0] =   ( _matrix[1][0] * _matrix[2][1] -
                                 _matrix[1][1] * _matrix[2][0] )
                               * det;
        out._matrix[0][1] = - ( _matrix[0][1] * _matrix[2][2] -
                                 _matrix[0][2] * _matrix[2][1] )
                               * det;
        out._matrix[1][1] =   ( _matrix[0][0] * _matrix[2][2] -
                                 _matrix[0][2] * _matrix[2][0] )
                               * det;
        out._matrix[2][1] = - ( _matrix[0][0] * _matrix[2][1] -
                                 _matrix[0][1] * _matrix[2][0] )
                               * det;
        out._matrix[0][2] =   ( _matrix[0][1] * _matrix[1][2] -
                                 _matrix[0][2] * _matrix[1][1] )
                               * det;
        out._matrix[1][2] = - ( _matrix[0][0] * _matrix[1][2] -
                                 _matrix[0][2] * _matrix[1][0] )
                               * det;
        out._matrix[2][2] =   ( _matrix[0][0] * _matrix[1][1] -
                                 _matrix[0][1] * _matrix[1][0] )
                               * det;

        /* Calculate -C * inverse(A) */
        out._matrix[3][0] = - ( _matrix[3][0] * out._matrix[0][0] +
                                 _matrix[3][1] * out._matrix[1][0] +
                                 _matrix[3][2] * out._matrix[2][0] );
        out._matrix[3][1] = - ( _matrix[3][0] * out._matrix[0][1] +
                                 _matrix[3][1] * out._matrix[1][1] +
                                 _matrix[3][2] * out._matrix[2][1] );
        out._matrix[3][2] = - ( _matrix[3][0] * out._matrix[0][2] +
                                 _matrix[3][1] * out._matrix[1][2] +
                                 _matrix[3][2] * out._matrix[2][2] );

        /* Fill in last column */
        out._matrix[0][3] = out._matrix[1][3] = out._matrix[2][3] = 0.0;
        out._matrix[3][3] = 1.0;
	}
	return(out);
}