
#ifndef CMATRIX_H
#define CMATRIX_H

#include "vectors.h"
#include <math.h>

#define EPSILON (1.0e-4)

class Matrix3x3
{
public:
	friend Matrix3x3 operator *( const Matrix3x3 & lhs, const Matrix3x3 rhs );
	friend Matrix3x3 operator *( const Matrix3x3 & lhs,  const float scaler );

public:
	Matrix3x3( void );
	Matrix3x3( const Matrix3x3 & rhs );
	Matrix3x3( float v00, float v01, float v02,
							float v10, float v11, float v12,
							float v20, float v21, float v22 );

	~Matrix3x3( void );
	
	Matrix3x3 & operator =( const Matrix3x3 & rhs );
	bool				operator ==( const Matrix3x3 & rhs );
	void				operator *=( const Matrix3x3 & rhs );
	void				operator *=( const float scaler );
	Matrix3x3   operator *( const float scaler );
	Vector		operator *( const Vector & v );
	
	void BuildRotation(Vector axis, float angle );
	float Determinant();
	Matrix3x3 Invert();
	Matrix3x3 Transpose();

	float _matrix[3][3];
};

class Matrix4x4
{
public:
	friend Matrix4x4 operator *( const Matrix4x4 & lhs, const Matrix4x4 rhs );
	friend Matrix4x4 operator *( const Matrix4x4 & lhs,  const int scaler );

public:
	Matrix4x4( void );
	Matrix4x4( const Matrix4x4 & rhs );
	Matrix4x4( float v00, float v01, float v02, float v03,
				float v10, float v11, float v12, float v13,
				float v20, float v21, float v22, float v23,
				float v30, float v31, float v32, float v33);

	~Matrix4x4( void );
	
	Matrix4x4 Inverse();
	Matrix4x4 & operator =( const Matrix4x4 & rhs );
	bool				operator ==( const Matrix4x4 & rhs );
	void				operator *=( const Matrix4x4 & rhs );
	//void				operator *=( const int scaler );
	//Matrix3x3   operator *( const int scaler );
	//Vector3D		operator *( const Vector3D & v );

	float _matrix[4][4];
};

#endif