#include <windows.h>
#include "scene.h"

Scene::Scene()
{
}

Scene::~Scene()
{
}
/*!
	Reads and parse the scene file.
	
	\param shape
		pointer to the shape

	\return bool
		false if the file does not exist, true otherwise

	\author
		Matt English
*/
bool Scene::OpenScene(char* fileName)
{
	std::string curLine, objectType, curNum;
	std::ifstream in;
	int i;
	ClearScene();
	ambient = Vector(0,0,0);
	air = Vector(-1, -1, -1);
	// Open the file
	in.open( fileName );
	if( !in.is_open() )
	{
		return (false);
	}
	//now we will read in the scene file and create objects based upon it
	while(in.eof() == false)
	{
		getline(in, curLine, '\n');
		objectType.clear();
		//we now have the line, and will find the type of object
		for(i = 0; curLine[i] != '\n' && curLine[i] != ' ' && curLine.length() > 0; ++i)
		{
			objectType += curLine[i];
		}
		if(objectType.compare("SPHERE") == 0)
		{
			//example: SPHERE (0.675,0,0) 0.175
			//	(0.25,0,0.75) 0.5 90 (0,0,0) 1e6 1
			Sphere temp;
			Vector center;
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			center.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			center.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			center.z = (float)atof(curNum.c_str());
			curNum.clear();
			//move it to the radius
			i+=2;
			temp.SetCenter(center);
			//now we go to the end of the line for the radius
			for(i; curLine[i] != '\n' && curLine[i] != '\0' && in.eof() == false; ++i)
				curNum+=curLine[i];
			temp.SetRadius((float)atof(curNum.c_str()));
			curNum.clear();
			//now we go to the next line
			getline(in, curLine, '\n');
			ReadSurfaceProperties(((BaseShape*)&temp), curLine, &in);
			sphereList.push_back(temp);
		}
		else if(objectType.compare("BOX") == 0)
		{
			//BOX (0,0,0) (0.241794,0.197593,-0.187063) (0.266726,0.0362919,-0.245031) (0.138063,0.132605,-0.309604)
			//	(0.5,0.5,0.3) 0.5 90 (0,0,0) 1e6 1
			Box box;
			Vector vtemp;
			//read in the corner
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			box.SetCorner(vtemp);

			//move it to the length
			i+=3;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			box.SetLength(vtemp);

			//move it to the width
			i+=3;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			box.SetWidth(vtemp);

			//move it to the length
			i+=3;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			box.SetHeight(vtemp);

			//get the surface properties
			getline(in, curLine, '\n');
			ReadSurfaceProperties(((BaseShape*)&box), curLine, &in);
			//calculate the normals
			box.CalculatePlanes();
			boxList.push_back(box);
		}
		else if(objectType.compare("POLYGON") == 0)
		{
			Poly poly;
			Vector vtemp;
			//read in the corner
			++i;
			for(i; curLine[i] != ' '; ++i)
			{
				curNum+=curLine[i];
			}
			poly.SetNumVerts(atoi(curNum.c_str()));
			curNum.clear();
			for(unsigned int count=0; count < poly.GetNumVerts(); ++count)
			{
				for(i; curLine[i] != '('; ++i)
				{
				}
				++i;
				for(i; curLine[i] != ','; ++i)
					curNum+=curLine[i];
				vtemp.x = (float)atof(curNum.c_str());
				curNum.clear();
				//move it one past the comma
				++i;
				for(i; curLine[i] != ','; ++i)
					curNum+=curLine[i];
				vtemp.y = (float)atof(curNum.c_str());
				curNum.clear();
				++i;
				for(i; curLine[i] != ')'; ++i)
					curNum+=curLine[i];
				vtemp.z = (float)atof(curNum.c_str());
				curNum.clear();
				poly.AddVert(vtemp);
			}
			getline(in, curLine, '\n');
			ReadSurfaceProperties(((BaseShape*)&poly), curLine, &in);
			polyList.push_back(poly);
		}
		else if(objectType.compare("ELLIPSOID") == 0)
		{
			Ellipsoid ellipse;
			Vector vtemp;

			for(i; curLine[i] != '('; ++i)
			{
			}
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			ellipse.SetCenter(vtemp);

			//move to the x semi axis
			for(i; curLine[i] != '('; ++i)
			{
			}
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			ellipse.SetAxisX(vtemp);

			//move to the y semi axis
			for(i; curLine[i] != '('; ++i)
			{
			}
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			ellipse.SetAxisY(vtemp);
			
			//move to the z semi axis
			for(i; curLine[i] != '('; ++i)
			{
			}
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			vtemp.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			vtemp.z = (float)atof(curNum.c_str());
			curNum.clear();
			ellipse.SetAxisZ(vtemp);

			//get the surface properties
			getline(in, curLine, '\n');
			ReadSurfaceProperties(((BaseShape*)&ellipse), curLine, &in);
			ellipse.CalculateAxisInverse();
			ellipsoidList.push_back(ellipse);
		}
		else if(objectType.compare("LIGHT") == 0)
		{
			//LIGHT (-1,-1,1) (0.5,0.5,0.5) 0.1
			Light tempLight;
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			tempLight.point.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			tempLight.point.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			tempLight.point.z = (float)atof(curNum.c_str());
			curNum.clear();

			//now we do the color
			for(i; curLine[i] != '('; ++i)
			{
			}
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			tempLight.color.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			tempLight.color.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			tempLight.color.z = (float)atof(curNum.c_str());
			curNum.clear();

			//now get the radius
			i+=2;
			for(i; curLine[i] != '\n' && curLine[i] != '\0' && in.eof() == false; ++i)
				curNum+=curLine[i];
			tempLight.radius = ((float)atof(curNum.c_str()));
			curNum.clear();
			lightList.push_back(tempLight);
		}
		else if(objectType.compare("LENS") == 0)
		{
			//example: SPHERE (0.675,0,0) 0.175
			//	(0.25,0,0.75) 0.5 90 (0,0,0) 1e6 1
			ConvexLens temp;
			Vector center;
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			temp.center1.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			temp.center1.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			temp.center1.z = (float)atof(curNum.c_str());
			curNum.clear();
			//move it to the radius
			i+=2;
			//now we go to the end of the line for the radius
			for(i; curLine[i] != '\n' && curLine[i] != ' ' && in.eof() == false; ++i)
				curNum+=curLine[i];
			temp.radius1=(float)atof(curNum.c_str());
			
			curNum.clear();
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			temp.center2.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			temp.center2.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			temp.center2.z = (float)atof(curNum.c_str());
			curNum.clear();
			//move it to the radius
			i+=2;
			//now we go to the end of the line for the radius
			for(i; curLine[i] != '\n' && curLine[i] != '\0' && in.eof() == false; ++i)
				curNum+=curLine[i];

			temp.radius2=(float)atof(curNum.c_str());
			curNum.clear();
			//now we go to the next line
			getline(in, curLine, '\n');
			ReadSurfaceProperties(((BaseShape*)&temp), curLine, &in);
			lensList.push_back(temp);
		}
		else if(objectType.compare("CAMERA") == 0)
		{
			//CAMERA (0.1,0.1,1) (0.5,0,0) (0,0.5,0) (0,0,1)
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			viewPlaneCenter.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			viewPlaneCenter.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			viewPlaneCenter.z = (float)atof(curNum.c_str());
			curNum.clear();

			//now we do the u vector
			i+=3;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			viewPlaneU.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			viewPlaneU.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			viewPlaneU.z = (float)atof(curNum.c_str());
			curNum.clear();

			//now we do the v vector
			i+=3;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			viewPlaneV.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			viewPlaneV.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			viewPlaneV.z = (float)atof(curNum.c_str());
			curNum.clear();

			//now we get the camera pos
			i+=3;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			camera.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			camera.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			camera.z = (float)atof(curNum.c_str());
			camera += viewPlaneCenter;
			curNum.clear();
		}
		else if(objectType.compare("AMBIENT") == 0)
		{
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			ambient.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			ambient.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			ambient.z = (float)atof(curNum.c_str());
			curNum.clear();
		}
		else if(objectType.compare("AIR") == 0)
		{
			for(i; curLine[i] != '('; ++i)
			{
			}
			//move it one past the left parenthese
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			air.x = (float)atof(curNum.c_str());
			curNum.clear();
			//move it one past the comma
			++i;
			for(i; curLine[i] != ','; ++i)
				curNum+=curLine[i];
			air.y = (float)atof(curNum.c_str());
			curNum.clear();
			++i;
			for(i; curLine[i] != ')'; ++i)
				curNum+=curLine[i];
			air.z = (float)atof(curNum.c_str());
			curNum.clear();
		}
	}
	return(true);
}
/*!
	Reads in the surface properties of a shape from the file
	
	\param shape
		pointer to the shape

	\param curLine
		the line of text containing the surface properties

	\param in
		pointer to the file stream for the scene file	

	\author
		Matt English
*/
void Scene::ReadSurfaceProperties(BaseShape* shape, std::string curLine, std::ifstream *in)
{
	Vector tempVector;
	int i;
	std::string curNum;

	for(i=0; curLine[i] != '('; ++i)
	{
	}
	++i;

	//we will now get the diffuse color
	for(i; curLine[i] != ','; ++i)
		curNum+=curLine[i];
	shape->diffusecolor.x = (float)atof(curNum.c_str());
	curNum.clear();
	//move it one past the comma
	++i;
	for(i; curLine[i] != ','; ++i)
		curNum+=curLine[i];
	shape->diffusecolor.y = (float)atof(curNum.c_str());
	curNum.clear();
	++i;
	for(i; curLine[i] != ')'; ++i)
		curNum+=curLine[i];
	shape->diffusecolor.z = (float)atof(curNum.c_str());
	curNum.clear();
	//move over the right paren
	i+=2;
	for(i; curLine[i] != ' '; ++i)
		curNum+=curLine[i];
	shape->specularReflectionCoef = (float)atof(curNum.c_str());
	curNum.clear();
	//move to specularReflectionExp
	++i;
	for(i; curLine[i] != ' '; ++i)
		curNum+=curLine[i];
	shape->specularReflectionExp = (float)atof(curNum.c_str());
	curNum.clear();
	//move past the left paren of the transAttenFactors
	i+=2;
	for(i; curLine[i] != ','; ++i)
		curNum+=curLine[i];
	tempVector.x = (float)atof(curNum.c_str());
	curNum.clear();
	++i;
	for(i; curLine[i] != ','; ++i)
		curNum+=curLine[i];
	tempVector.y = (float)atof(curNum.c_str());
	curNum.clear();
	++i;
	for(i; curLine[i] != ')'; ++i)
		curNum+=curLine[i];
	tempVector.z = (float)atof(curNum.c_str());
	curNum.clear();
	shape->transAttenFactors = tempVector;
	//move to the relElecPermittivity
	i+=2;
	for(i; curLine[i] != ' '; ++i)
		curNum+=curLine[i];
	shape->relElecPermittivity = (float)atof(curNum.c_str());
	curNum.clear();
	//move to the relMagnPermeability
	++i;
	for(i; curLine[i] != '\n' && curLine[i] != '\0' && in->eof() == false; ++i)
		curNum+=curLine[i];
	shape->relMagnPermeability = (float)atof(curNum.c_str());

	shape->refractionIndex = sqrt(shape->relElecPermittivity * shape->relMagnPermeability);
	curNum.clear();
}
/*!
	Clears the scene of all objects

	\author
		Matt English
*/
void Scene::ClearScene()
{
	ambient = Vector(0,0,0);
	air = Vector(-1, -1, -1);
	sphereList.clear();
	boxList.clear();
	polyList.clear();
	ellipsoidList.clear();
	lightList.clear();
	lensList.clear();
}