#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include "vectors.h"

#define tol 0.0000001f

float Vector::Length()
{
	float temp=x*x + y*y + z*z;
	assert(temp >= 0);
	return  ((float)sqrt(temp));
}
float Vector::LengthSq()
{
	return(x*x + y*y + z*z);
}
//
Vector Vector::Normalize()
{
	Vector v;
	float mag = Length();

	if ( mag <= tol )
		mag = 1.f;

	v.x = x / mag;
	v.y = y / mag;
	v.z = z / mag;
	if( fabs( v.x ) < tol ) v.x = 0.f;
	if( fabs( v.y ) < tol ) v.y = 0.f;
	if( fabs( v.z ) < tol ) v.z = 0.f;
	return v;
}
//
Vector& Vector::Normal()
{
	float temp;
	temp=this->x;
	this->x=-this->y;
	this->y=temp;
	return *this;

}
//
Vector Vector::NormalNoMod()
{
	float temp;
	Vector v;
	v.x=this->x;
	v.y=this->y;
	temp=v.x;
	v.x=-v.y;
	v.y=temp;
	v.z=0;
	return(v);
}
//
Vector Vector::operator -(void)
{
	return Vector( -x, -y, -z );
}
//
Vector Vector::operator +(Vector v)
{
	Vector k;
	k.x = this->x + v.x;
	k.y = this->y + v.y;
	k.z = this->z + v.z;
	return k;
}
//
Vector Vector::operator -(Vector v)
{
	Vector k;
	k.x = this->x - v.x;
	k.y = this->y - v.y;
	k.z = this->z - v.z;
	return k;
}
//
Vector& Vector::operator += (Vector v)
{
	this->x += v.x;
	this->y += v.y;
	this->z += v.z;
	return *this;
}
//
Vector& Vector::operator += (float f)
{
	this->x += f;
	this->y += f;
	this->z += f;
	return *this;
}
//
Vector& Vector::operator -= (Vector v)
{
	this->x -= v.x;
	this->y -= v.y;
	this->z -= v.z;
	return *this;
}
//
Vector Vector::operator * (float s)
{
	Vector v;
	v.x = this->x * s;
	v.y = this->y * s;
	v.z = this->z * s;
	return v;
}
//
Vector Vector::operator / (float s)
{
	Vector v;
	v.x = this->x / s;
	v.y = this->y / s;
	v.z = this->z / s;
	return v;
}
//
//dot product
float Vector::operator * (Vector v)
{
	return (float)(this->x*v.x + this->y*v.y + this->z*v.z);
}
//
Vector Vector::operator ^(Vector v)
{
	Vector crossed;
	crossed.x = (y * v.z)-(v.y * z);
	crossed.y = -((x * v.z)-(v.x * z));
	crossed.z = (x * v.y)-(v.x * y);
	return(crossed);
}
//


